import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_entry.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/log/log_entry/log_entry.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:flutter/material.dart';

class LogScreen extends StatefulWidget {
  static const String routeName = '/log';
  const LogScreen({Key? key}) : super(key: key);

  @override
  _LogScreenState createState() => _LogScreenState();
}

class _LogScreenState extends State<LogScreen> {
  late Map<DateTime, List<LogEntry>> _logEntryDailyMap;

  @override
  void initState() {
    super.initState();
    reload();
  }

  void reload({String? message}) {
    setState(() {
      _logEntryDailyMap = LogEntry.getDailyEntryMap();
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void onDelete(LogEntry logEntry) {
    LogEntry.remove(logEntry.id);
    reload(message: 'Log Entry deleted');
  }

  void handleDeleteAction(LogEntry logEntry) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(logEntry),
        message: 'Are you sure you want to delete this Log Entry?',
      );
    } else {
      onDelete(logEntry);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Log Entries'),
        actions: <Widget>[
          IconButton(
            onPressed: reload,
            icon: const Icon(Icons.refresh)
          ),
        ],
      ),
      drawer: const Navigation(currentLocation: LogScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: _logEntryDailyMap.isNotEmpty ? SingleChildScrollView(
              child: ListView.builder(
                shrinkWrap: true,
                padding: const EdgeInsets.all(10.0),
                itemCount: _logEntryDailyMap.length,
                itemBuilder: (context, dateIndex) {
                  List<DateTime> dateList = _logEntryDailyMap.keys.toList();
                  final date = dateList[dateIndex];
                  final entryList = _logEntryDailyMap[date];
                  return ListBody(
                    children: [
                      Text(DateTimeUtils.displayDate(date)),
                      entryList != null && entryList.isNotEmpty
                        ? ListView.builder(
                          shrinkWrap: true,
                          itemCount: entryList.length,
                          itemBuilder: (context, index) {
                            final logEntry = entryList[index];
                            return ListTile(
                              onTap: () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) =>
                                        LogEntryScreen(
                                            id: logEntry.id),
                                  ),
                                ).then((message) => reload(
                                    message: message));
                              },
                              title: Text(
                                  DateTimeUtils.displayTime(
                                      logEntry.time)),
                              // ignore: todo
                              // TODO: add more data (event, glucose color...)
                              subtitle: Text(logEntry
                                          .mgPerDl !=
                                      null
                                        ? '${logEntry.mgPerDl.toString()} mg/dl'
                                        : ''),
                                    trailing: Row(
                                      mainAxisSize:
                                          MainAxisSize.min,
                                      children: [
                                        IconButton(
                                          onPressed: () =>
                                              handleDeleteAction(
                                                  logEntry),
                                          icon: const Icon(
                                              Icons.delete,
                                              color: Colors.blue),
                                        )
                                      ],
                                    ),
                                  );
                                }
                        ) : Container(),
                    ], 
                  );
                },
              ),
            ) : const Center(
                child: Text('You have not created any Log Entries yet!'),
              ),
          ),
        ],
      ),
      // ignore: todo
      // TODO: add button for active events
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => const LogEntryScreen(),
            ),
          ).then((message) => reload(message: message));
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
