import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/dropdown.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/accuracy.dart';
import 'package:diameter/models/log_meal.dart';
import 'package:diameter/models/meal.dart';
import 'package:diameter/models/meal_category.dart';
import 'package:diameter/models/meal_portion_type.dart';
import 'package:diameter/models/meal_source.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/settings.dart';
import 'package:diameter/utils/utils.dart';
import 'package:flutter/material.dart';

class LogMealDetailScreen extends StatefulWidget {
  static const String routeName = '/log-meal';

  final int logEntryId;
  final int id;

  const LogMealDetailScreen({Key? key, this.logEntryId = 0, this.id = 0})
      : super(key: key);

  @override
  _LogMealDetailScreenState createState() => _LogMealDetailScreenState();
}

class _LogMealDetailScreenState extends State<LogMealDetailScreen> {
  LogMeal? _logMeal;
  bool _isNew = true;
  bool _isSaving = false;

  final GlobalKey<FormState> _logMealForm = GlobalKey<FormState>();

  final _valueController = TextEditingController(text: '');
  final _carbsRatioController = TextEditingController(text: '');
  final _portionSizeController = TextEditingController(text: '');
  final _carbsPerPortionController = TextEditingController(text: '');
  final _bolusController = TextEditingController(text: '');
  final _delayedBolusRateController = TextEditingController(text: '');
  final _delayedBolusDurationController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');

  Meal? _meal;
  MealSource? _mealSource;
  MealCategory? _mealCategory;
  MealPortionType? _mealPortionType;
  Accuracy? _portionSizeAccuracy;
  Accuracy? _carbsRatioAccuracy;

  List<Meal> _meals = [];
  List<MealCategory> _mealCategories = [];
  List<MealPortionType> _mealPortionTypes = [];
  List<MealSource> _mealSources = [];
  List<Accuracy> _portionSizeAccuracies = [];
  List<Accuracy> _carbsRatioAccuracies = [];

  @override
  void initState() {
    super.initState();
    reload();

    _portionSizeAccuracies = Accuracy.getAllForPortionSize();
    _carbsRatioAccuracies = Accuracy.getAllForCarbsRatio();
    _meals = Meal.getAll();
    _mealCategories = MealCategory.getAll();
    _mealPortionTypes = MealPortionType.getAll();
    _mealSources = MealSource.getAll();

    if (widget.id != 0) {
      _valueController.text = _logMeal!.value;
      _carbsRatioController.text = (_logMeal!.carbsRatio ?? '').toString();
      _portionSizeController.text = (_logMeal!.portionSize ?? '').toString();
      _carbsPerPortionController.text =
          (_logMeal!.carbsPerPortion ?? '').toString();
      _bolusController.text = (_logMeal!.bolus ?? '').toString();
      _delayedBolusRateController.text =
          (_logMeal!.delayedBolusRate ?? '').toString();
      _delayedBolusDurationController.text =
          (_logMeal!.delayedBolusDuration ?? '').toString();
      _notesController.text = _logMeal!.notes ?? '';
    }
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _logMeal = LogMeal.get(widget.id);
      });
    }
    _isNew = _logMeal == null;
  }

  Future<void> onSelectMeal(Meal meal) async {
    setState(() {
      _meal = meal;
      _valueController.text = meal.value;
      if (meal.carbsRatio != null) {
        _carbsRatioController.text = meal.carbsRatio.toString();
      }
      if (meal.portionSize != null) {
        _portionSizeController.text = meal.portionSize.toString();
      }
      if (meal.carbsPerPortion != null) {
        _carbsPerPortionController.text = meal.carbsPerPortion.toString();
      }
      if (meal.delayedBolusRate != null) {
        _delayedBolusRateController.text = meal.delayedBolusRate.toString();
      }
      if (meal.delayedBolusDuration != null) {
        _delayedBolusDurationController.text =
            meal.delayedBolusDuration.toString();
      }
      if (meal.mealSource.hasValue) {
        _mealSource = meal.mealSource.target;
      }
      if (meal.mealCategory.hasValue) {
        _mealCategory = meal.mealCategory.target;
      }
      if (meal.mealPortionType.hasValue) {
        _mealPortionType = meal.mealPortionType.target;
      }
      if (meal.portionSizeAccuracy.hasValue) {
        _portionSizeAccuracy = meal.portionSizeAccuracy.target;
      }
      if (meal.carbsRatioAccuracy.hasValue) {
        _carbsRatioAccuracy = meal.carbsRatioAccuracy.target;
      }
    });
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_logMealForm.currentState!.validate()) {
      LogMeal logMeal = LogMeal(
        id: widget.id,
        value: _valueController.text,
        carbsRatio: double.tryParse(_carbsRatioController.text),
        portionSize: double.tryParse(_portionSizeController.text),
        carbsPerPortion: double.tryParse(_carbsPerPortionController.text),
        bolus: double.tryParse(_bolusController.text),
        delayedBolusDuration:
            int.tryParse(_delayedBolusDurationController.text),
        delayedBolusRate: double.tryParse(_delayedBolusRateController.text),
        notes: _notesController.text,
      );
      logMeal.logEntry.targetId = widget.logEntryId;
      logMeal.meal.target = _meal;
      logMeal.mealSource.target = _mealSource;
      logMeal.mealCategory.target = _mealCategory;
      logMeal.mealPortionType.target = _mealPortionType;
      logMeal.portionSizeAccuracy.target = _portionSizeAccuracy;
      logMeal.carbsRatioAccuracy.target = _carbsRatioAccuracy;

      LogMeal.put(logMeal);
      Navigator.pop(context, '${_isNew ? 'New' : ''} Meal Saved');
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    if (showConfirmationDialogOnCancel &&
        ((_isNew &&
                (_valueController.text != '' ||
                    _meal != null ||
                    _mealSource != null ||
                    _mealCategory != null ||
                    _mealPortionType != null ||
                    double.tryParse(_carbsRatioController.text) != null ||
                    double.tryParse(_portionSizeController.text) != null ||
                    double.tryParse(_carbsPerPortionController.text) != null ||
                    _carbsRatioAccuracy != null ||
                    _portionSizeAccuracy != null ||
                    double.tryParse(_bolusController.text) != null ||
                    int.tryParse(_delayedBolusDurationController.text) !=
                        null ||
                    double.tryParse(_delayedBolusRateController.text) != null ||
                    _notesController.text != '')) ||
            (!_isNew &&
                (_valueController.text != _logMeal!.value ||
                    _meal != _logMeal!.meal.target ||
                    _mealSource != _logMeal!.mealSource.target ||
                    _mealCategory != _logMeal!.mealCategory.target ||
                    _mealPortionType != _logMeal!.mealPortionType.target ||
                    double.tryParse(_carbsRatioController.text) !=
                        _logMeal!.carbsRatio ||
                    double.tryParse(_portionSizeController.text) !=
                        _logMeal!.portionSize ||
                    double.tryParse(_carbsPerPortionController.text) !=
                        _logMeal!.carbsPerPortion ||
                    _carbsRatioAccuracy !=
                        _logMeal!.carbsRatioAccuracy.target ||
                    _portionSizeAccuracy !=
                        _logMeal!.portionSizeAccuracy.target ||
                    double.tryParse(_bolusController.text) != _logMeal!.bolus ||
                    int.tryParse(_delayedBolusDurationController.text) !=
                        _logMeal!.delayedBolusDuration ||
                    double.tryParse(_delayedBolusRateController.text) !=
                        _logMeal!.delayedBolusRate ||
                    _notesController.text != (_logMeal!.notes ?? ''))))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  void calculateThirdMeasurementOfPortionCarbsRelation(
      {PortionCarbsParameter? parameterToBeCalculated}) {
    double? carbsRatio;
    double? portionSize;
    double? carbsPerPortion;

    if (parameterToBeCalculated != PortionCarbsParameter.carbsRatio &&
        _carbsRatioController.text != '') {
      carbsRatio = double.tryParse(_carbsRatioController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.portionSize &&
        _portionSizeController.text != '') {
      portionSize = double.tryParse(_portionSizeController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.carbsPerPortion &&
        _carbsRatioController.text != '') {
      carbsPerPortion = double.tryParse(_carbsPerPortionController.text);
    }

    if (carbsRatio != null && portionSize != null && carbsPerPortion == null) {
      setState(() {
        _carbsPerPortionController.text =
            Utils.calculateCarbsPerPortion(carbsRatio!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio == null && portionSize != null && carbsPerPortion != null) {
      setState(() {
        _carbsRatioController.text =
            Utils.calculateCarbsRatio(carbsPerPortion!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio != null && portionSize == null && carbsPerPortion != null) {
      setState(() {
        _portionSizeController.text =
            Utils.calculatePortionSize(carbsRatio!, carbsPerPortion!)
                .toString();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Meal' : _logMeal!.value),
      ),
      drawer: const Navigation(currentLocation: LogMealDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            FormWrapper(
              formState: _logMealForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                AutoCompleteDropdownButton<Meal>(
                  selectedItem: _meal,
                  label: 'Meal',
                  items: _meals,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    if (value != null) {
                      onSelectMeal(value);
                    }
                  },
                ),
                AutoCompleteDropdownButton<MealSource>(
                  selectedItem: _mealSource,
                  label: 'Meal Source',
                  items: _mealSources,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _mealSource = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<MealCategory>(
                  selectedItem: _mealCategory,
                  label: 'Meal Category',
                  items: _mealCategories,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _mealCategory = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<MealPortionType>(
                  selectedItem: _mealPortionType,
                  label: 'Meal Portion Type',
                  items: _mealPortionTypes,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _mealPortionType = value;
                    });
                  },
                ),
                Row(
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: const InputDecoration(
                          labelText: 'Carbs ratio',
                          suffixText: '%',
                        ),
                        controller: _carbsRatioController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.carbsRatio),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: InputDecoration(
                          labelText: 'Portion size',
                          suffixText:
                              nutritionMeasurement == NutritionMeasurement.grams
                                  ? 'g'
                                  : nutritionMeasurement ==
                                          NutritionMeasurement.ounces
                                      ? 'oz'
                                      : '',
                          alignLabelWithHint: true,
                        ),
                        controller: _portionSizeController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.portionSize),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _portionSizeAccuracy,
                  label: 'Portion Size Accuracy',
                  items: _portionSizeAccuracies,
                  // getItemValue: (item) => item.objectId,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _portionSizeAccuracy = value;
                    });
                  },
                ),
                Row(
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: InputDecoration(
                          labelText: 'Carbs per portion',
                          suffixText:
                              nutritionMeasurement == NutritionMeasurement.grams
                                  ? 'g'
                                  : nutritionMeasurement ==
                                          NutritionMeasurement.ounces
                                      ? 'oz'
                                      : '',
                        ),
                        controller: _carbsPerPortionController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.carbsPerPortion),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _carbsRatioAccuracy,
                  label: 'Carbs Ratio Accuracy',
                  items: _carbsRatioAccuracies,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _carbsRatioAccuracy = value;
                    });
                  },
                ),
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Bolus Units',
                    suffixText: ' U',
                  ),
                  controller: _bolusController,
                  keyboardType:
                      const TextInputType.numberWithOptions(decimal: true),
                ),
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Delayed Bolus Duration',
                    suffixText: ' min',
                  ),
                  controller: _delayedBolusDurationController,
                  keyboardType: const TextInputType.numberWithOptions(),
                ),
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Delayed Bolus Units',
                    suffixText: ' U',
                    alignLabelWithHint: true,
                  ),
                  controller: _delayedBolusRateController,
                  keyboardType:
                      const TextInputType.numberWithOptions(decimal: true),
                ),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                ),
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
