import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/meal.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/meal/meal_detail.dart';
import 'package:flutter/material.dart';

class MealListScreen extends StatefulWidget {
  static const String routeName = '/meals';

  const MealListScreen({Key? key}) : super(key: key);

  @override
  _MealListScreenState createState() => _MealListScreenState();
}

class _MealListScreenState extends State<MealListScreen> {
  List<Meal> _meals = [];

  @override
  void initState() {
    super.initState();
    reload();
  }

  void reload({String? message}) {
    setState(() {
      _meals = Meal.getAll();
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void onDelete(Meal meal) {
    Meal.remove(meal.id);
    reload(message: 'Meal deleted');
  }

  void handleDeleteAction(Meal meal) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(meal),
        message: 'Are you sure you want to delete this Meal?',
      );
    } else {
      onDelete(meal);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Meals'), actions: <Widget>[
        IconButton(onPressed: reload, icon: const Icon(Icons.refresh))
      ]),
      drawer: const Navigation(currentLocation: MealListScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: _meals.isNotEmpty ? ListView.builder(
              padding: const EdgeInsets.all(10.0),
              itemCount: _meals.length,
              itemBuilder: (context, index) {
                final meal = _meals[index];

                return ListTile(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) =>
                            MealDetailScreen(id: meal.id),
                      ),
                    ).then((message) => reload(message: message));
                  },
                  title: Text(meal.value),
                  subtitle: Text(meal.notes ?? ''),
                  trailing: Row(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      IconButton(
                        onPressed: () => handleDeleteAction(meal),
                        icon: const Icon(Icons.delete,
                            color: Colors.blue),
                      )
                    ],
                  ),
                );
              },
            ): const Center(
              child: Text('You have not created any Meals yet!'),
            ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => const MealDetailScreen(),
            ),
          ).then((message) => reload(message: message));
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
